#!/usr/bin/python
#
# Copyright (C) 2008 Jacques Supcik
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


"""aspsms provides an interface to the aspsms.com services
  
  references: http://www.aspsms.com/xml/doc/xmlsvr191.pdf

  ASPSMS: Main Class
"""

__author__ = 'Jacques Supcik'

import httplib
import random
import re
import xml.dom.minidom
from warnings import warn
from aspsms.aspxml import aspxml

def _sms_encode(s):
    return re.sub(
        r'[\x26\x3c\x3e\x80-\xff]',
        lambda c: '&#'+str(ord(c.group(0)))+';',
        unicode(s, 'utf-8').encode("iso8859_15")
    )


class ASPSMS(object):
    """Main Class
    """

    #Default values for members
    reply = None

    def __init__(self, user_key=None, password=None, originator=None):
        """Creates a new ASMSMS object"""

        self.user_key = user_key
	self.password = password
	self.originator = originator

    def reply_attribute(self, attribute):
        """"Extracts an attribute from the reply"""

	i=self.reply.getElementsByTagName(attribute).item(0)
	return (i != None and i.firstChild.data) or None

    def __send(self, message):
        """Sends the message"""

        urls = [
            "xml1.aspsms.com:5061", "xml1.aspsms.com:5098",
            "xml2.aspsms.com:5061", "xml2.aspsms.com:5098",
        ]
        random.shuffle(urls)

        while urls:
            try:
                url = urls.pop(0)
                conn = httplib.HTTPConnection(url)
                conn.request("POST", "/xmlsvr.asp", str(message))
                r = conn.getresponse()
                reply = r.read()
                if r.status == 200:
                    break
                warn("Error % (%)" % (r.status, r.reason))
            except Exception, inst:
                warn("%s / error using %s" % (inst, url))
  
        self.reply = xml.dom.minidom.parseString(reply)
	self.error_code = self.reply_attribute("ErrorCode")
	self.error_description = self.reply_attribute("ErrorDescription")

    def send_text_sms(self, recipient, text, originator=None):
        """Sends a text sms

        Args:
          originator: str
          recipient: str
          text: str
        """

        originator = originator or self.originator
        message = aspxml(self.user_key, self.password)
        if originator:
            message.tag('Originator', originator)
        message.open_tag('Recipient')
        message.tag('PhoneNumber', recipient)
        message.tag('UsedCredits', 1)
        message.close_tag()
        message.tag('MessageData', _sms_encode(text.strip()))
        message.tag('Action', 'SendTextSMS')
        self.__send(message)

    def show_credits(self):
        """Show credits

        Returns:
          the remaining credits.
        """

        message = aspxml(self.user_key, self.password)
        message.tag("Action", "ShowCredits")
        self.__send(message)
	return self.reply_attribute("Credits")




#!/usr/bin/python
#
# Copyright (C) 2008 Jacques Supcik
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

__author__ = 'Jacques Supcik'

class aspxml(object):
    # Note: I wanted to call this class just "xml", but then, I would not
    # have been able to use "xml.dom.minidom" in the api class. I could
    # have solved this issue by importing "absolute_import" from "__future__",
    # but this would have restricted this "py-aspsms" to python 2.5.
    def __init__(self, userkey=None, password=None):
        self.stack = []
        self.xml = ''
        if userkey != None:
            self.tag("Userkey", userkey)
        if password != None:
            self.tag("Password", password)


    def open_tag(self, tag):
        self.xml += "  " * len(self.stack) + "<%s>\n" % tag
        self.stack.insert(-1, tag)

    def close_tag(self, tag=None):
        if tag == None:
            tag = self.stack.pop()
        else:
            self.stack.pop()
        self.xml += "  " * len(self.stack) + "</%s>\n" % tag

    def tag(self, tag, value=''):
        self.xml += "  " * len(self.stack) + "<%s>%s</%s>\n" % \
                (tag, value, tag)

    def __str__(self):
        return '<?xml version="1.0" encoding="ISO-8859-1"?>\n' + \
               '<aspsms>\n' + self.xml + '</aspsms>'




#!/usr/bin/python
#
# Copyright (C) 2008 Jacques Supcik
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import optparse
import os
import sys

def exit(sms, debug=False):
    if debug:
        print "Debug: return=%s (%s)" % (sms.error_code, sms.error_description)
        print "---BEGIN REPLY---"
        print sms.reply.toprettyxml(),
        print "---END REPLY---"
        
    code = int(sms.error_code)
    if code == 1:
        sys.exit(0)
    elif code == 0:
        print "Error: %s" % sms.error_description
        sys.exit(-1)
    else:
        print "Error: %s" % sms.error_description
        sys.exit(code)


class OptionParser(optparse.OptionParser):

    def __init__(self):
        optparse.OptionParser.__init__(self)
        self.add_option("-u", "--userkey", dest="userkey",
            default=os.environ.get("ASPSMS_USERKEY", None),
            help="user key",
        )
        self.add_option("-p", "--password", dest="password",
            default=os.environ.get("ASPSMS_PASSWORD", None),
            help="password",
        )
        self.add_option("--debug", dest="debug",
            default=os.environ.get("ASPSMS_DEBUG", False),
            action="store_true", help="Debug"
        )

    def add_originator_option(self):
        self.add_option("-o", "--originator", dest="originator",
              default=os.environ.get("ASPSMS_ORIGINATOR", None))

    def add_recipient_option(self):
        self.add_option("-r", "--recipient", dest="recipient",
              default=os.environ.get("ASPSMS_RECIPIENT", None))




#!/usr/bin/env python

import sys
from distutils.core import setup

if sys.hexversion < 0x02040000:
    raise RuntimeError, "Python 2.4.0 or higher required"

setup(name='py-aspsms',
    version='0.1.2',
    description='py-aspsms: Python interface to the aspsms.com services',
    long_description='''
aspsms (www.aspsms.com) is a Swiss company that offers SMS gateway
services for a fair price. Py-aspsms is a Python interface for this
service. Currently, only basic operations are supported (e.g. sending
a simple SMS or getting information about the remaining credit) but I
am currently extending it. If you have specific neeeds (e.g. sending a
ringtone or an operator logo) just put your request on the issue tracker
or send me an e-mail.
'''.strip(),
    author='Jacques Supcik',
    author_email='jacques@supcik.org',
    url='http://code.google.com/p/py-aspsms/',
    download_url='http://code.google.com/p/py-aspsms/downloads/list',
    scripts=['send_sms', 'show_sms_credit'],
    packages=['aspsms'],
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Intended Audience :: Telecommunications Industry',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Communications',
        'Topic :: Internet',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Utilities',
    ],
)

